@@ -3,7 +3,7 @@
 /*
     Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2020 Particular Programs Ltd.
+    Copyright 2007-2021 Particular Programs Ltd.
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -21,7 +21,7 @@
     you must obtain a valid commercial licence before doing so.
 */
 
-#include "rubberband/RubberBandStretcher.h"
+#include "../rubberband/RubberBandStretcher.h"
 
 #include <iostream>
 #include <sndfile.h>
@@ -33,29 +33,32 @@
 
 #include <fstream>
 
-#include "system/sysutils.h"
+#include "../src/system/sysutils.h"
 
-#ifdef __MSVC__
-#include "getopt/getopt.h"
+#ifdef _MSC_VER
+#include "../src/getopt/getopt.h"
 #else
 #include <getopt.h>
 #include <unistd.h>
 #include <sys/time.h>
 #endif
 
-#include "base/Profiler.h"
-
-using namespace std;
-using namespace RubberBand;
+#include "../src/base/Profiler.h"
 
 #ifdef _WIN32
 using RubberBand::gettimeofday;
 #endif
 
-#ifdef __MSVC__
+#ifdef _MSC_VER
 using RubberBand::usleep;
+#define strdup _strdup
 #endif
 
+using RubberBand::RubberBandStretcher;
+
+using std::cerr;
+using std::endl;
+
 double tempo_convert(const char *str)
 {
     char *d = strchr((char *)str, ':');
@@ -104,7 +107,10 @@ int main(int argc, char **argv)
 
     bool haveRatio = false;
 
-    std::string mapfile;
+    std::string timeMapFile;
+    std::string freqMapFile;
+    std::string pitchMapFile;
+    bool freqOrPitchMapSpecified = false;
 
     enum {
         NoTransients,
@@ -118,6 +124,8 @@ int main(int argc, char **argv)
         SoftDetector
     } detector = CompoundDetector;
 
+    bool ignoreClipping = false;
+
     while (1) {
         int optionIndex = 0;
 
@@ -150,6 +158,9 @@ int main(int argc, char **argv)
             { "threads",       0, 0, '@' },
             { "quiet",         0, 0, 'q' },
             { "timemap",       1, 0, 'M' },
+            { "freqmap",       1, 0, 'Q' },
+            { "pitchmap",      1, 0, 'C' },
+            { "ignore-clipping", 0, 0, 'i' },
             { 0, 0, 0, 0 }
         };
 
@@ -170,7 +181,7 @@ int main(int argc, char **argv)
         case 'R': realtime = true; break;
         case 'L': precise = false; break;
         case 'P': precise = true; break;
-	case 'F': formant = true; break;
+        case 'F': formant = true; break;
         case '0': threading = 1; break;
         case '@': threading = 2; break;
         case '1': transients = NoTransients; crispchanged = true; break;
@@ -185,7 +196,10 @@ int main(int argc, char **argv)
         case '%': hqpitch = true; break;
         case 'c': crispness = atoi(optarg); break;
         case 'q': quiet = true; break;
-        case 'M': mapfile = optarg; break;
+        case 'M': timeMapFile = optarg; break;
+        case 'Q': freqMapFile = optarg; freqOrPitchMapSpecified = true; break;
+        case 'C': pitchMapFile = optarg; freqOrPitchMapSpecified = true; break;
+        case 'i': ignoreClipping = true; break;
         default:  help = true; break;
         }
     }
@@ -195,11 +209,20 @@ int main(int argc, char **argv)
         return 0;
     }
 
+    if (freqOrPitchMapSpecified) {
+        if (freqMapFile != "" && pitchMapFile != "") {
+            cerr << "ERROR: Please specify either pitch map or frequency map, not both" << endl;
+            return 1;
+        }
+        haveRatio = true;
+        realtime = true;
+    }
+    
     if (help || !haveRatio || optind + 2 != argc) {
         cerr << endl;
 	cerr << "Rubber Band" << endl;
         cerr << "An audio time-stretching and pitch-shifting library and utility program." << endl;
-	cerr << "Copyright 2007-2020 Particular Programs Ltd." << endl;
+	cerr << "Copyright 2007-2021 Particular Programs Ltd." << endl;
         cerr << endl;
 	cerr << "   Usage: " << argv[0] << " [options] <infile.wav> <outfile.wav>" << endl;
         cerr << endl;
@@ -213,23 +236,45 @@ int main(int argc, char **argv)
         cerr << "  -p<X>, --pitch <X>      Raise pitch by X semitones, or" << endl;
         cerr << "  -f<X>, --frequency <X>  Change frequency by multiple X" << endl;
         cerr << endl;
-        cerr << "  -M<F>, --timemap <F>    Use file F as the source for key frame map" << endl;
+        cerr << "The following options provide ways of making the time and frequency ratios" << endl;
+        cerr << "change during the audio." << endl;
         cerr << endl;
-        cerr << "A map file consists of a series of lines each having two numbers separated" << endl;
-        cerr << "by a single space.  These are source and target sample frame numbers for fixed" << endl;
-        cerr << "time points within the audio data, defining a varying stretch factor through" << endl;
-        cerr << "the audio.  You must specify an overall stretch factor using e.g. -t as well." << endl;
+        cerr << "  -M<F>, --timemap <F>    Use file F as the source for time map" << endl;
         cerr << endl;
-        cerr << "The following options provide a simple way to adjust the sound.  See below" << endl;
+        cerr << "  A time map (or key-frame map) file contains a series of lines, each with two" << endl;
+        cerr << "  sample frame numbers separated by a single space. These are source and" << endl;
+        cerr << "  target frames for fixed time points within the audio data, defining a varying" << endl;
+        cerr << "  stretch factor through the audio. When supplying a time map you must specify" << endl;
+        cerr << "  an overall stretch factor using -t, -T, or -D as well, to determine the" << endl;
+        cerr << "  total output duration." << endl;
+        cerr << endl;
+        cerr << "         --pitchmap <F>   Use file F as the source for pitch map" << endl;
+        cerr << endl;
+        cerr << "  A pitch map file contains a series of lines, each with two values: the input" << endl;
+        cerr << "  sample frame number and a pitch offset in semitones, separated by a single" << endl;
+        cerr << "  space. These specify a varying pitch factor through the audio. The offsets" << endl;
+        cerr << "  are all relative to an initial offset specified by the pitch or frequency" << endl;
+        cerr << "  option, or relative to no shift if neither was specified. Offsets are" << endl;
+        cerr << "  not cumulative. This option implies realtime mode (-R) and also enables a" << endl;
+        cerr << "  high-consistency pitch shifting mode, appropriate for dynamic pitch changes." << endl;
+        cerr << "  Because of the use of realtime mode, the overall duration will not be exact." << endl;
+        cerr << endl;
+        cerr << "         --freqmap <F>    Use file F as the source for frequency map" << endl;
+        cerr << endl;
+        cerr << "  A frequency map file is like a pitch map, except that its second column" << endl;
+        cerr << "  lists frequency multipliers rather than pitch offsets (like the difference" << endl;
+        cerr << "  between pitch and frequency options above)." << endl;
+        cerr << endl;
+        cerr << "The following options provide a simple way to adjust the sound. See below" << endl;
         cerr << "for more details." << endl;
         cerr << endl;
         cerr << "  -c<N>, --crisp <N>      Crispness (N = 0,1,2,3,4,5,6); default 5 (see below)" << endl;
-	cerr << "  -F,    --formant        Enable formant preservation when pitch shifting" << endl;
+        cerr << "  -F,    --formant        Enable formant preservation when pitch shifting" << endl;
         cerr << endl;
         cerr << "The remaining options fine-tune the processing mode and stretch algorithm." << endl;
         cerr << "These are mostly included for test purposes; the default settings and standard" << endl;
         cerr << "crispness parameter are intended to provide the best sounding set of options" << endl;
-        cerr << "for most situations.  The default is to use none of these options." << endl;
+        cerr << "for most situations. The default is to use none of these options." << endl;
         cerr << endl;
         cerr << "  -L,    --loose          Relax timing in hope of better transient preservation" << endl;
         cerr << "  -P,    --precise        Ignored: The opposite of -L, this is default from 1.6" << endl;
@@ -247,6 +292,8 @@ int main(int argc, char **argv)
         cerr << "         --pitch-hq       In RT mode, use a slower, higher quality pitch shift" << endl;
         cerr << "         --centre-focus   Preserve focus of centre material in stereo" << endl;
         cerr << "                          (at a cost in width and individual channel quality)" << endl;
+        cerr << "         --ignore-clipping Ignore clipping at output; the default is to restart" << endl;
+        cerr << "                          with reduced gain if clipping occurs" << endl;
         cerr << endl;
         cerr << "  -d<N>, --debug <N>      Select debug level (N = 0,1,2,3); default 0, full 3" << endl;
         cerr << "                          (N.B. debug level 3 includes audible ticks in output)" << endl;
@@ -264,7 +311,7 @@ int main(int argc, char **argv)
         cerr << "  -c 5   default processing options" << endl;
         cerr << "  -c 6   equivalent to --no-lamination --window-short (may be good for drums)" << endl;
         cerr << endl;
-	return 2;
+        return 2;
     }
 
     if (ratio <= 0.0) {
@@ -277,6 +324,12 @@ int main(int argc, char **argv)
         cerr << "         provided -- crispness will override these other options" << endl;
     }
 
+    if (hqpitch && freqOrPitchMapSpecified) {
+        cerr << "WARNING: High-quality pitch mode selected, but frequency or pitch map file is" << endl;
+        cerr << "         provided -- pitch mode will be overridden by high-consistency mode" << endl;
+        hqpitch = false;
+    }
+
     switch (crispness) {
     case -1: crispness = 5; break;
     case 0: detector = CompoundDetector; transients = NoTransients; lamination = false; longwin = true; shortwin = false; break;
@@ -302,34 +355,35 @@ int main(int argc, char **argv)
         cerr << ")" << endl;
     }
 
-    std::map<size_t, size_t> mapping;
-    
-    if (mapfile != "") {
-        std::ifstream ifile(mapfile.c_str());
+    std::map<size_t, size_t> timeMap;
+    if (timeMapFile != "") {
+        std::ifstream ifile(timeMapFile.c_str());
         if (!ifile.is_open()) {
-            cerr << "ERROR: Failed to open time map file \"" << mapfile << "\""
-                 << endl;
+            cerr << "ERROR: Failed to open time map file \""
+                 << timeMapFile << "\"" << endl;
             return 1;
         }
         std::string line;
         int lineno = 0;
         while (!ifile.eof()) {
             std::getline(ifile, line);
-            while (line.length() > 0 && line[0] == ' ') line = line.substr(1);
+            while (line.length() > 0 && line[0] == ' ') {
+                line = line.substr(1);
+            }
             if (line == "") {
                 ++lineno;
                 continue;
             }
             std::string::size_type i = line.find_first_of(" ");
             if (i == std::string::npos) {
-                cerr << "ERROR: Time map file \"" << mapfile
+                cerr << "ERROR: Time map file \"" << timeMapFile
                      << "\" is malformed at line " << lineno << endl;
                 return 1;
             }
             size_t source = atoi(line.substr(0, i).c_str());
             while (i < line.length() && line[i] == ' ') ++i;
             size_t target = atoi(line.substr(i).c_str());
-            mapping[source] = target;
+            timeMap[source] = target;
             if (debug > 0) {
                 cerr << "adding mapping from " << source << " to " << target << endl;
             }
@@ -338,7 +392,57 @@ int main(int argc, char **argv)
         ifile.close();
 
         if (!quiet) {
-            cerr << "Read " << mapping.size() << " line(s) from map file" << endl;
+            cerr << "Read " << timeMap.size() << " line(s) from time map file" << endl;
+        }
+    }
+
+    std::map<size_t, double> freqMap;
+
+    if (freqOrPitchMapSpecified) {
+        std::string file = freqMapFile;
+        bool convertFromPitch = false;
+        if (pitchMapFile != "") {
+            file = pitchMapFile;
+            convertFromPitch = true;
+        }
+        std::ifstream ifile(file.c_str());
+        if (!ifile.is_open()) {
+            cerr << "ERROR: Failed to open map file \"" << file << "\"" << endl;
+            return 1;
+        }
+        std::string line;
+        int lineno = 0;
+        while (!ifile.eof()) {
+            std::getline(ifile, line);
+            while (line.length() > 0 && line[0] == ' ') {
+                line = line.substr(1);
+            }
+            if (line == "") {
+                ++lineno;
+                continue;
+            }
+            std::string::size_type i = line.find_first_of(" ");
+            if (i == std::string::npos) {
+                cerr << "ERROR: Map file \"" << file
+                     << "\" is malformed at line " << lineno << endl;
+                return 1;
+            }
+            size_t source = atoi(line.substr(0, i).c_str());
+            while (i < line.length() && line[i] == ' ') ++i;
+            double freq = atof(line.substr(i).c_str());
+            if (convertFromPitch) {
+                freq = pow(2.0, freq / 12.0);
+            }
+            freqMap[source] = freq;
+            if (debug > 0) {
+                cerr << "adding mapping for source frame " << source << " of frequency multiplier " << freq << endl;
+            }
+            ++lineno;
+        }
+        ifile.close();
+
+        if (!quiet) {
+            cerr << "Read " << freqMap.size() << " line(s) from frequency map file" << endl;
         }
     }
 
@@ -354,9 +458,9 @@ int main(int argc, char **argv)
 
     sndfile = sf_open(fileName, SFM_READ, &sfinfo);
     if (!sndfile) {
-	cerr << "ERROR: Failed to open input file \"" << fileName << "\": "
-	     << sf_strerror(sndfile) << endl;
-	return 1;
+        cerr << "ERROR: Failed to open input file \"" << fileName << "\": "
+             << sf_strerror(sndfile) << endl;
+        return 1;
     }
 
     if (sfinfo.samplerate == 0) {
@@ -382,13 +486,10 @@ int main(int argc, char **argv)
     
     sndfileOut = sf_open(fileNameOut, SFM_WRITE, &sfinfoOut) ;
     if (!sndfileOut) {
-	cerr << "ERROR: Failed to open output file \"" << fileNameOut << "\" for writing: "
-	     << sf_strerror(sndfileOut) << endl;
-	return 1;
+        cerr << "ERROR: Failed to open output file \"" << fileNameOut << "\" for writing: "
+             << sf_strerror(sndfileOut) << endl;
+        return 1;
     }
-    
-    int ibs = 1024;
-    size_t channels = sfinfo.channels;
 
     RubberBandStretcher::Options options = 0;
     if (realtime)    options |= RubberBandStretcher::OptionProcessRealTime;
@@ -398,9 +499,14 @@ int main(int argc, char **argv)
     if (shortwin)    options |= RubberBandStretcher::OptionWindowShort;
     if (smoothing)   options |= RubberBandStretcher::OptionSmoothingOn;
     if (formant)     options |= RubberBandStretcher::OptionFormantPreserved;
-    if (hqpitch)     options |= RubberBandStretcher::OptionPitchHighQuality;
     if (together)    options |= RubberBandStretcher::OptionChannelsTogether;
 
+    if (freqOrPitchMapSpecified) {
+        options |= RubberBandStretcher::OptionPitchHighConsistency;
+    } else if (hqpitch) {
+        options |= RubberBandStretcher::OptionPitchHighQuality;
+    }
+    
     switch (threading) {
     case 0:
         options |= RubberBandStretcher::OptionThreadingAuto;
@@ -438,56 +544,267 @@ int main(int argc, char **argv)
     }
 
     if (pitchshift != 0.0) {
-        frequencyshift *= pow(2.0, pitchshift / 12);
+        frequencyshift *= pow(2.0, pitchshift / 12.0);
     }
 
     cerr << "Using time ratio " << ratio;
-    cerr << " and frequency ratio " << frequencyshift << endl;
 
+    if (!freqOrPitchMapSpecified) {
+        cerr << " and frequency ratio " << frequencyshift << endl;
+    } else {
+        cerr << " and initial frequency ratio " << frequencyshift << endl;
+    }
+    
 #ifdef _WIN32
     RubberBand::
 #endif
     timeval tv;
     (void)gettimeofday(&tv, 0);
-
+    
     RubberBandStretcher::setDefaultDebugLevel(debug);
 
-    RubberBandStretcher ts(sfinfo.samplerate, channels, options,
-                           ratio, frequencyshift);
+    size_t countIn = 0, countOut = 0;
 
-    ts.setExpectedInputDuration(sfinfo.frames);
+    float gain = 1.f;
+    bool successful = false;
+    
+    const size_t channels = sfinfo.channels;
+    const int bs = 1024;
+    
+    float **cbuf = new float *[channels];
+    for (size_t c = 0; c < channels; ++c) {
+        cbuf[c] = new float[bs];
+    }
+    float *ibuf = new float[channels * bs];
 
-    float *fbuf = new float[channels * ibs];
-    float **ibuf = new float *[channels];
-    for (size_t i = 0; i < channels; ++i) ibuf[i] = new float[ibs];
+    int thisBlockSize;
 
-    int frame = 0;
-    int percent = 0;
+    while (!successful) { // we may have to repeat with a modified
+                          // gain, if clipping occurs
+        successful = true;
 
-    sf_seek(sndfile, 0, SEEK_SET);
+        RubberBandStretcher ts(sfinfo.samplerate, channels, options,
+                               ratio, frequencyshift);
+        ts.setExpectedInputDuration(sfinfo.frames);
 
-    if (!realtime) {
+        int frame = 0;
+        int percent = 0;
 
-        if (!quiet) {
-            cerr << "Pass 1: Studying..." << endl;
+        sf_seek(sndfile, 0, SEEK_SET);
+
+        if (!realtime) {
+
+            if (!quiet) {
+                cerr << "Pass 1: Studying..." << endl;
+            }
+
+            while (frame < sfinfo.frames) {
+
+                int count = -1;
+                if ((count = sf_readf_float(sndfile, ibuf, bs)) <= 0) break;
+        
+                for (size_t c = 0; c < channels; ++c) {
+                    for (int i = 0; i < count; ++i) {
+                        cbuf[c][i] = ibuf[i * channels + c];
+                    }
+                }
+
+                bool final = (frame + bs >= sfinfo.frames);
+
+                ts.study(cbuf, count, final);
+
+                int p = int((double(frame) * 100.0) / sfinfo.frames);
+                if (p > percent || frame == 0) {
+                    percent = p;
+                    if (!quiet) {
+                        cerr << "\r" << percent << "% ";
+                    }
+                }
+
+                frame += bs;
+            }
+
+            if (!quiet) {
+                cerr << "\rCalculating profile..." << endl;
+            }
+
+            sf_seek(sndfile, 0, SEEK_SET);
         }
 
+        frame = 0;
+        percent = 0;
+
+        if (!timeMap.empty()) {
+            ts.setKeyFrameMap(timeMap);
+        }
+
+        std::map<size_t, double>::const_iterator freqMapItr = freqMap.begin();
+    
+        countIn = 0;
+        countOut = 0;
+        bool clipping = false;
+
+        // The stretcher only pads the start in offline mode; to avoid
+        // a fade in at the start, we pad it manually in RT mode
+        int toDrop = 0;
+        if (realtime) {
+            toDrop = int(ts.getLatency());
+            int toPad = int(round(toDrop * frequencyshift));
+            if (debug > 0) {
+                cerr << "padding start with " << toPad
+                     << " samples in RT mode, will drop " << toDrop
+                     << " at output" << endl;
+            }
+            if (toPad > 0) {
+                for (size_t c = 0; c < channels; ++c) {
+                    for (int i = 0; i < bs; ++i) {
+                        cbuf[c][i] = 0.f;
+                    }
+                }
+                while (toPad > 0) {
+                    int p = toPad;
+                    if (p > bs) p = bs;
+                    ts.process(cbuf, p, false);
+                    toPad -= p;
+                }
+            }
+        }                
+        
         while (frame < sfinfo.frames) {
 
-            int count = -1;
+            thisBlockSize = bs;
 
-            if ((count = sf_readf_float(sndfile, fbuf, ibs)) <= 0) break;
-        
-            for (size_t c = 0; c < channels; ++c) {
-                for (int i = 0; i < count; ++i) {
-                    float value = fbuf[i * channels + c];
-                    ibuf[c][i] = value;
+            while (freqMapItr != freqMap.end()) {
+                size_t nextFreqFrame = freqMapItr->first;
+                if (nextFreqFrame <= countIn) {
+                    double s = frequencyshift * freqMapItr->second;
+                    if (debug > 0) {
+                        cerr << "at frame " << countIn
+                             << " (requested at " << freqMapItr->first
+                             << " [NOT] plus latency " << ts.getLatency()
+                             << ") updating frequency ratio to " << s << endl;
+                    }
+                    ts.setPitchScale(s);
+                    ++freqMapItr;
+                } else {
+                    if (nextFreqFrame < countIn + thisBlockSize) {
+                        thisBlockSize = nextFreqFrame - countIn;
+                    }
+                    break;
                 }
             }
 
-            bool final = (frame + ibs >= sfinfo.frames);
+            int count = -1;
+            if ((count = sf_readf_float(sndfile, ibuf, thisBlockSize)) < 0) {
+                break;
+            }
+        
+            countIn += count;
 
-            ts.study(ibuf, count, final);
+            for (size_t c = 0; c < channels; ++c) {
+                for (int i = 0; i < count; ++i) {
+                    cbuf[c][i] = ibuf[i * channels + c];
+                }
+            }
+
+            bool final = (frame + thisBlockSize >= sfinfo.frames);
+
+            if (debug > 2) {
+                cerr << "count = " << count << ", bs = " << thisBlockSize << ", frame = " << frame << ", frames = " << sfinfo.frames << ", final = " << final << endl;
+            }
+
+            ts.process(cbuf, count, final);
+
+            int avail;
+            while ((avail = ts.available()) > 0) {
+                if (debug > 1) {
+                    cerr << "available = " << avail << endl;
+                }
+
+                thisBlockSize = avail;
+                if (thisBlockSize > bs) {
+                    thisBlockSize = bs;
+                }
+                
+                if (toDrop > 0) {
+                    int dropHere = toDrop;
+                    if (dropHere > thisBlockSize) {
+                        dropHere = thisBlockSize;
+                    }
+                    if (debug > 1) {
+                        cerr << "toDrop = " << toDrop << ", dropping "
+                             << dropHere << " of " << avail << endl;
+                    }
+                    ts.retrieve(cbuf, dropHere);
+                    toDrop -= dropHere;
+                    avail -= dropHere;
+                    continue;
+                }
+                
+                if (debug > 2) {
+                    cerr << "retrieving block of " << thisBlockSize << endl;
+                }
+                ts.retrieve(cbuf, thisBlockSize);
+                
+                if (realtime && final) {
+                    // (in offline mode the stretcher handles this itself)
+                    size_t ideal = size_t(countIn * ratio);
+                    if (debug > 2) {
+                        cerr << "at end, ideal = " << ideal
+                             << ", countOut = " << countOut
+                             << ", thisBlockSize = " << thisBlockSize << endl;
+                    }
+                    if (countOut + thisBlockSize > ideal) {
+                        thisBlockSize = ideal - countOut;
+                        if (debug > 1) {
+                            cerr << "truncated final block to " << thisBlockSize
+                                 << endl;
+                        }
+                    }
+                }
+                
+                countOut += thisBlockSize;
+                
+                for (size_t c = 0; c < channels; ++c) {
+                    for (int i = 0; i < thisBlockSize; ++i) {
+                        float value = gain * cbuf[c][i];
+                        if (ignoreClipping) { // i.e. just clamp, don't bail out
+                            if (value > 1.f) value = 1.f;
+                            if (value < -1.f) value = -1.f;
+                        } else {
+                            if (value >= 1.f || value < -1.f) {
+                                clipping = true;
+                                gain = (0.999f / fabsf(cbuf[c][i]));
+                            }
+                        }
+                        ibuf[i * channels + c] = value;
+                    }
+                }
+                sf_writef_float(sndfileOut, ibuf, thisBlockSize);
+            }
+
+            if (clipping) {
+                if (!quiet) {
+                    cerr << "NOTE: Clipping detected at output sample "
+                         << countOut << ", restarting with "
+                         << "reduced gain of " << gain
+                         << " (supply --ignore-clipping to avoid this)" << endl;
+                }
+                const float mingain = 0.75f;
+                if (gain < mingain) {
+                    cerr << "WARNING: Clipped values were implausibly high: "
+                         << "something wrong with input or process - "
+                         << "not reducing gain below " << mingain << endl;
+                    gain = mingain;
+                    ignoreClipping = true;
+                }
+                successful = false;
+                break;
+            }
+            
+            if (frame == 0 && !realtime && !quiet) {
+                cerr << "Pass 2: Processing..." << endl;
+            }
 
             int p = int((double(frame) * 100.0) / sfinfo.frames);
             if (p > percent || frame == 0) {
@@ -497,139 +814,69 @@ int main(int argc, char **argv)
                 }
             }
 
-            frame += ibs;
+            frame += count;
         }
 
-        if (!quiet) {
-            cerr << "\rCalculating profile..." << endl;
+        if (!successful) {
+            sf_seek(sndfile, 0, SEEK_SET);
+            sf_seek(sndfileOut, 0, SEEK_SET);
+            continue;
         }
-
-        sf_seek(sndfile, 0, SEEK_SET);
-    }
-
-    frame = 0;
-    percent = 0;
-
-    if (!mapping.empty()) {
-        ts.setKeyFrameMap(mapping);
-    }
     
-    size_t countIn = 0, countOut = 0;
-
-    while (frame < sfinfo.frames) {
-
-        int count = -1;
-
-	if ((count = sf_readf_float(sndfile, fbuf, ibs)) < 0) break;
-        
-        countIn += count;
-
-        for (size_t c = 0; c < channels; ++c) {
-            for (int i = 0; i < count; ++i) {
-                float value = fbuf[i * channels + c];
-                ibuf[c][i] = value;
-            }
+        if (!quiet) {
+            cerr << "\r    " << endl;
         }
 
-        bool final = (frame + ibs >= sfinfo.frames);
-
-        if (debug > 2) {
-            cerr << "count = " << count << ", ibs = " << ibs << ", frame = " << frame << ", frames = " << sfinfo.frames << ", final = " << final << endl;
-        }
-
-        ts.process(ibuf, count, final);
-
-        int avail = ts.available();
-        if (debug > 1) cerr << "available = " << avail << endl;
-
-        if (avail > 0) {
-            float **obf = new float *[channels];
-            for (size_t i = 0; i < channels; ++i) {
-                obf[i] = new float[avail];
+        int avail;
+        while ((avail = ts.available()) >= 0) {
+            if (debug > 1) {
+                cerr << "(completing) available = " << avail << endl;
             }
-            ts.retrieve(obf, avail);
-            countOut += avail;
-            float *fobf = new float[channels * avail];
-            for (size_t c = 0; c < channels; ++c) {
-                for (int i = 0; i < avail; ++i) {
-                    float value = obf[c][i];
-                    if (value > 1.f) value = 1.f;
-                    if (value < -1.f) value = -1.f;
-                    fobf[i * channels + c] = value;
+
+            if (avail == 0) {
+                if (realtime ||
+                    (options & RubberBandStretcher::OptionThreadingNever)) {
+                    break;
+                } else {
+                    usleep(10000);
                 }
             }
-//            cout << "fobf mean: ";
-//    double d = 0;
-//    for (int i = 0; i < avail; ++i) {
-//        d += fobf[i];
-//    }
-//    d /= avail;
-//    cout << d << endl;
-            sf_writef_float(sndfileOut, fobf, avail);
-            delete[] fobf;
-            for (size_t i = 0; i < channels; ++i) {
-                delete[] obf[i];
+            
+            thisBlockSize = avail;
+            if (thisBlockSize > bs) {
+                thisBlockSize = bs;
             }
-            delete[] obf;
-        }
+                
+            ts.retrieve(cbuf, thisBlockSize);
 
-        if (frame == 0 && !realtime && !quiet) {
-            cerr << "Pass 2: Processing..." << endl;
-        }
-
-	int p = int((double(frame) * 100.0) / sfinfo.frames);
-	if (p > percent || frame == 0) {
-	    percent = p;
-            if (!quiet) {
-                cerr << "\r" << percent << "% ";
-            }
-	}
-
-        frame += ibs;
-    }
-
-    if (!quiet) {
-        cerr << "\r    " << endl;
-    }
-    int avail;
-
-    while ((avail = ts.available()) >= 0) {
-
-        if (debug > 1) {
-            cerr << "(completing) available = " << avail << endl;
-        }
-
-        if (avail > 0) {
-            float **obf = new float *[channels];
-            for (size_t i = 0; i < channels; ++i) {
-                obf[i] = new float[avail];
-            }
-            ts.retrieve(obf, avail);
-            countOut += avail;
-            float *fobf = new float[channels * avail];
+            countOut += thisBlockSize;
+                
             for (size_t c = 0; c < channels; ++c) {
-                for (int i = 0; i < avail; ++i) {
-                    float value = obf[c][i];
+                for (int i = 0; i < thisBlockSize; ++i) {
+                    float value = gain * cbuf[c][i];
                     if (value > 1.f) value = 1.f;
                     if (value < -1.f) value = -1.f;
-                    fobf[i * channels + c] = value;
+                    ibuf[i * channels + c] = value;
                 }
             }
-
-            sf_writef_float(sndfileOut, fobf, avail);
-            delete[] fobf;
-            for (size_t i = 0; i < channels; ++i) {
-                delete[] obf[i];
-            }
-            delete[] obf;
-        } else {
-            usleep(10000);
+                
+            sf_writef_float(sndfileOut, ibuf, thisBlockSize);
         }
     }
 
+    delete[] ibuf;
+
+    for (size_t c = 0; c < channels; ++c) {
+        delete[] cbuf[c];
+    }
+    delete[] cbuf;
+
     sf_close(sndfile);
     sf_close(sndfileOut);
 
+    free(fileName);
+    free(fileNameOut);
+
     if (!quiet) {
 
         cerr << "in: " << countIn << ", out: " << countOut << ", ratio: " << float(countOut)/float(countIn) << ", ideal output: " << lrint(countIn * ratio) << ", error: " << abs(lrint(countIn * ratio) - int(countOut)) << endl;
@@ -637,7 +884,7 @@ int main(int argc, char **argv)
 #ifdef _WIN32
         RubberBand::
 #endif
-        timeval etv;
+            timeval etv;
         (void)gettimeofday(&etv, 0);
         
         etv.tv_sec -= tv.tv_sec;
@@ -652,7 +899,7 @@ int main(int argc, char **argv)
     }
 
     RubberBand::Profiler::dump();
-
+    
     return 0;
 }
 
