@@ -3,7 +3,7 @@
 /*
     Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2020 Particular Programs Ltd.
+    Copyright 2007-2021 Particular Programs Ltd.
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -21,13 +21,20 @@
     you must obtain a valid commercial licence before doing so.
 */
 
-#ifndef _RUBBERBANDSTRETCHER_H_
-#define _RUBBERBANDSTRETCHER_H_
+#ifndef RUBBERBAND_STRETCHER_H
+#define RUBBERBAND_STRETCHER_H
     
-#define RUBBERBAND_VERSION "1.9.0"
+#define RUBBERBAND_VERSION "2.0.0"
 #define RUBBERBAND_API_MAJOR_VERSION 2
 #define RUBBERBAND_API_MINOR_VERSION 6
 
+#undef RUBBERBAND_DLLEXPORT
+#ifdef _MSC_VER
+#define RUBBERBAND_DLLEXPORT __declspec(dllexport)
+#else
+#define RUBBERBAND_DLLEXPORT
+#endif
+
 #include <vector>
 #include <map>
 #include <cstddef>
@@ -37,7 +44,22 @@
  * 
  * The Rubber Band API is contained in the single class
  * RubberBand::RubberBandStretcher.
- * 
+ *
+ * The Rubber Band stretcher supports two processing modes, offline
+ * and real-time. The choice of mode is fixed on construction. In
+ * offline mode, you must provide the audio block-by-block in two
+ * passes: in the first pass calling study(), in the second pass
+ * calling process() and receiving the output via retrieve(). In
+ * real-time mode, there is no study pass, just a single streaming
+ * pass in which the audio is passed to process() and output received
+ * via retrieve().
+ *
+ * In real-time mode you can change the time and pitch ratios at any
+ * time, but in offline mode they are fixed and cannot be changed
+ * after the study pass has begun. (However, see setKeyFrameMap() for
+ * a way to do pre-planned variable time stretching in offline mode.)
+ * Offline mode typically produces slightly more precise results.
+ *
  * Threading notes for real-time applications:
  * 
  * Multiple instances of RubberBandStretcher may be created and used
@@ -60,7 +82,8 @@
 namespace RubberBand
 {
 
-class RubberBandStretcher
+class RUBBERBAND_DLLEXPORT
+RubberBandStretcher
 {
 public:
     /**
@@ -178,7 +201,7 @@ public:
      *   one processing thread per audio channel in offline mode if
      *   the stretcher is able to determine that more than one CPU is
      *   available, and one thread only in realtime mode.  This is the
-     *   defafult.
+     *   default.
      *
      *   \li \c OptionThreadingNever - Never use more than one thread.
      *  
@@ -310,7 +333,7 @@ public:
         OptionPitchHighConsistency = 0x04000000,
 
         OptionChannelsApart        = 0x00000000,
-        OptionChannelsTogether     = 0x10000000,
+        OptionChannelsTogether     = 0x10000000
 
         // n.b. Options is int, so we must stop before 0x80000000
     };
@@ -324,11 +347,28 @@ public:
 
     /**
      * Construct a time and pitch stretcher object to run at the given
-     * sample rate, with the given number of channels.  Processing
-     * options and the time and pitch scaling ratios may be provided.
-     * The time and pitch ratios may be changed after construction,
-     * but most of the options may not.  See the option documentation
-     * above for more details.
+     * sample rate, with the given number of channels.
+     *
+     * Initial time and pitch scaling ratios and other processing
+     * options may be provided. In particular, the behaviour of the
+     * stretcher depends strongly on whether offline or real-time mode
+     * is selected on construction (via OptionProcessOffline or
+     * OptionProcessRealTime option - offline is the default).
+     * 
+     * In offline mode, you must provide the audio block-by-block in
+     * two passes: in the first pass calling study(), in the second
+     * pass calling process() and receiving the output via
+     * retrieve(). In real-time mode, there is no study pass, just a
+     * single streaming pass in which the audio is passed to process()
+     * and output received via retrieve().
+     *
+     * In real-time mode you can change the time and pitch ratios at
+     * any time, but in offline mode they are fixed and cannot be
+     * changed after the study pass has begun. (However, see
+     * setKeyFrameMap() for a way to do pre-planned variable time
+     * stretching in offline mode.)
+     *
+     * See the option documentation above for more details.
      */
     RubberBandStretcher(size_t sampleRate,
                         size_t channels,
@@ -456,7 +496,7 @@ public:
     /**
      * Change an OptionPitch configuration setting.  This may be
      * called at any time in RealTime mode.  It may not be called in
-     * Offline mode (for which the transients option is fixed on
+     * Offline mode (for which the pitch option is fixed on
      * construction).
      */
     void setPitchOption(Options options);
@@ -627,8 +667,10 @@ public:
 
     /**
      * Obtain some processed output data from the stretcher.  Up to
-     * "samples" samples will be stored in the output arrays (one per
-     * channel for de-interleaved audio data) pointed to by "output".
+     * "samples" samples will be stored in each of the output arrays
+     * (one per channel for de-interleaved audio data) pointed to by
+     * "output".  The number of sample frames available to be
+     * retrieved can be queried beforehand with a call to available().
      * The return value is the actual number of sample frames
      * retrieved.
      *
