@@ -23,7 +23,6 @@
 
 #include "StretcherImpl.h"
 
-using namespace std;
 
 namespace RubberBand {
 
@@ -122,7 +121,7 @@ RubberBandStretcher::setMaxProcessSize(size_t samples)
 }
 
 void
-RubberBandStretcher::setKeyFrameMap(const map<size_t, size_t> &mapping)
+RubberBandStretcher::setKeyFrameMap(const std::map<size_t, size_t> &mapping)
 {
     m_d->setKeyFrameMap(mapping);
 }
@@ -177,19 +176,19 @@ RubberBandStretcher::getInputIncrement() const
     return m_d->getInputIncrement();
 }
 
-vector<int>
+std::vector<int>
 RubberBandStretcher::getOutputIncrements() const
 {
     return m_d->getOutputIncrements();
 }
 
-vector<float>
+std::vector<float>
 RubberBandStretcher::getPhaseResetCurve() const
 {
     return m_d->getPhaseResetCurve();
 }
 
-vector<int>
+std::vector<int>
 RubberBandStretcher::getExactTimePoints() const
 {
     return m_d->getExactTimePoints();
