@@ -23,8 +23,8 @@
 
 #include "SpectralDifferenceAudioCurve.h"
 
-#include "system/Allocators.h"
-#include "system/VectorOps.h"
+#include "../system/Allocators.h"
+#include "../system/VectorOps.h"
 
 namespace RubberBand
 {
